# devlop

字段打包部署项目重构

# 项目构成
- 后端项目 node+express+mysql 提供restful接口
- 前端项目 vue+ivews 提供显示层


# INSTALL

## 后端项目

```shell
$ git clone http://10.100.11.5/zhouzihao/node-devlop-rest-api.git 
$ cd node-devlop-rest-api.git 
$ npm install
$ node main.js
```
## 前端项目
```
$ git clone http://10.100.11.5/zhouzihao/deploy-ui.git
$ cd deploy-ui
$ yarn install
$ yarn run serve
## build with  $ yarn build 
```

# 配置项

## 后端项目

### config >> common.js
```javascript
const config = {
    host: '10.16.11.106', // 部署机器的域名
    port: 3000  // 暴露本服务的端口号
}
```

### config >> db.js
数据库配置。
```javascript
const dbConfig = {
    host:"mysql.docker.local", 
    username:"root",
    password:"Zz123456zZ",
    database:"devlop"
}
```


## 前端项目
### src >> config >>configs.js
```javascript
const serve_url = "http://127.0.0.1:3000"   //后端想的服务地址
```


# 使用说明
项目，环境，配置进行了分离配置。  
环境是独立的。在环境列表里配了打包的必要信息。包括要运行gitlab CI的目标地址。还有要打包到的镜像仓库的地址。  
项目只关注项目git源地址。和项目类型。（项目类型在平台源数据中配置）还有项目的源数据进行配置。  
如你所见构建和项目和环境也是隔离。在构建时要选择要构建的环境和指定分支和版本号。

> 注意！如无需要，平台源数据请不要改动。

## 项目源数据
项目源数据默认可以不填。它告诉程序该如何扫描包和识别到Jar包。默认没有的情况下，程序会在项目根目录下找包。
它的配置语法如下。
```json
{
    "module_list":[
        "app-a",
        "app-b"
    ]
}
```
上面标识要去`app-a`和`app-b`两个目录下找Jar包。jar包的名称也是默认读下面pom文件中的模块名去识别的。如果你要修改jar的读取规则可以使用下面的写法：
```json
{
    "module_list":[
        {
            "name":"app-a",
            "exec-file":"app-a-*-bb"
        },
        "app-b"
    ]
}
```
使用`exec-file`来修改这个规则，可以使用通配符，而且在这个列表中你可以直接写字符串，他的作业和写一个Object只配置name字段是等效的。

> 注意 exec-file 参数应该只能识别到一个文件。


## 配置中心

在项目列表中在对应项目的操作栏有一个⚙图标。通过这个方法进入项目的配置中心。配置是针对项目-环境的。也就是说每个环境对于同一个项目也是独立的。  
同一个环境同一个项目下的配置可以建立互不影响的任意条。 每一条配置都有一个地址。这是你要操作文件相对于项目的地址。  
在替换方式上有两种。提供和覆盖。  

- 提供。 使用json的map.把key替换成value.全局模式。
- 覆盖。 直接使用配置内容全部覆盖文件内容。


## 构建项目

在项目列表的右侧的操作栏是有build按钮。点击后进入选择构建参数的页面。参数很少但是很重要包括：

- 分支： 要构建源项目地址的那个分支。
- 版本： 我们自行维护的一个版本。其验证规则是在平台源数据中。
- 环境:   要构建到的环境。可以在环境的预设值中选择。

构建开始后。会在目标的gitlab中建立一个项目。项目名是原来环境id+项目名。镜像也使用这种格式进行命名。  
一旦构建后按钮显示成功后，可以到目标的gitlab的具体项目中查看构建的详细日志。

> 注意 同一个项目，在同一个环境中，进行中的项目只能有一个，不然会直接报错。如果出现特殊情况，请进入数据中手动处理数据。

## 平台源数据

保存一些构建中的重要数据。比如dockerfile和CICD的原始模板文件。建议只有开发扩展和改变规则时才去碰他们。  
比如上文中修改版本的验证规则。可以在Java的ci模板中。修改tag的正则过滤规则

```
/^(r|t)-([0-9]+).([0-9]+).([0-9]+)/   只匹配 r-1.1.1或者 t-11.1.2 这种类型

/^v-([0-9]+).([0-9]+).([0-9]+)-.([0-9]+/  只匹配 v-1.1.1-12
```


## [新特性] 部署测试环境
在build中有是否要部署的选项。如果勾选后可以选择要部署的模块列表。（这个数据来自项目）
在测试环境中只有被选中的模块才会更新。但是构建时docker的最新版本都会更新。  
这里使用的原理是docker swarm中的update命令。
```shell
docker service update --image {image_name} {service_name}
```
 这个service_name是服务更新的关键。可以在项目源数据里配置。如果不配置有默认情况。下面将讲详细介绍：

 ###  配置模式（推荐）：
 在项目第一次建立之初。要手动启动服务并且指定服务名。
 在项目配置源数据时可以指定模块在环境中的服务名方便直接建立。
 ```json
 {
     "module_list":[
         {
             "name":"a",
             "exec-file":"a-*-b",
             "deploy":{
                 "dev":"[这里是服务名]"
             }
         }
     ]
 }
 ```

 注意模块必选使用这种方式进行配置。在deploy中是环境和环境名的键值对。也就说只有配置了对应环境的环境名才能正常的部署。
 

 ###  默认模式：
 如果系统读不到这个值的话。则会生成一个默认的名字。

 ```
 【环境名】-【模块名】
 ```

 > 环境准备。在对应的dockerSwarm机器的master机器上要注册一个可以运行docker命令的runner。而且要和定义的环境名一致。


# 关于部署

 项目部署在测试环境。地址：http://10.100.9.45/  
 后端项目使用pm2进行部署。前端项目使用ng的docker进行搭建。

## 后端项目
在跟目录里node-devlop-rest-api。
- 启动： `pm2 start main,js`
- 关闭：`pm2 stop 0` ,`pm2 del 0`

## 前端项目
在跟目录里的devlop-ui项目中
1. 构建项目 `yarn build` .项目会生成文件夹dist
2. dist 移动到根目录改名为 `iview-layoutui`
3. 在docker/config/app中docker-compose的配置文件。关闭启动都可以


# TODO

- 接入权限

